# Written by Pradeep Kyasanur
# This file was used for simulation with static scenario
# There is an accompanying sample.csh which may be used to setup simulations
# ======================================================================
# Define options
# ======================================================================
set opt(chan)		Channel/WirelessChannel
#set opt(prop)		Propagation/TwoRayGround

# Use shadowing channel model for more realistic modeling of the channel
set opt(prop)		Propagation/Shadowing
set opt(netif)		Phy/WirelessPhy 
set opt(mac)		Mac/802_11
set opt(macmb)          Mac/802_11mb
set opt(ifq)		Queue/DropTail/PriQueue
set opt(ll)		LL
set opt(ant)            Antenna/OmniAntenna

set opt(x)	        1500	;# X dimension of the topography
set opt(y)		750	;# Y dimension of the topography
#set opt(cp)		"./cbr"
#set opt(sc)		"./scen-670x670-10-0-2-1"

set opt(ifqlen)		50		;# max packet in ifq

# Uncomment the following lines if u don't want to give the parameter values
# from command line
set opt(nn)		9   ;# number of nodes per base station including it
set opt(misbnum)        1   ;# number of misbehaving nodes
set opt(baseX)          750
set opt(baseY)          350
set opt(errorRate)     0

# Type of simulation, SIMPLE is just 802.11, NO_MISB is modified scheme in absence of
# misbehavior, MISB_NOCORRECT is with misbehavior using detection scheme, but correction scheme is not
# enabled, MISB_CORRECT is modified scheme with misbehavior and correction enabled
#set opt(simType)        SIMPLE
#set opt(simType)        NO_MISB
#set opt(simType)        MISB_NOCORRECT
set opt(simType)        MISB_CORRECT

# Base is base station type of mode, ADHOC is random placement of node
set opt(networkType)    BASE
#set opt(networkType)    ADHOC

# number of other nodes. Set to 0 if 0 background flows are present, 2 if one
# background flow is present, 4 if two background flows are present
set opt(otherNodes)     2         

#Protocol parameters
set opt(JITTER)        1
set opt(THRESH)        5
set opt(K)             20

# MISB_PERC is unfortunately opposite of what was used in the paper. Smaller values imply greater misbehavior
set opt(MISB_PERC)    20
set opt(backoff_policy) 2 ;# If this  number is greater than 2, this number is used as the value for decrementing
                          ;# misb variable, and adaptive misbehavior strategy is enabled


set opt(distance)       150   ;#distance between nodes
set opt(rate)   	2000000
set opt(otherRate)      100000
set opt(LOG_DEV)        10.0

set opt(seed)		1

set opt(start)		0
set opt(stop)          	50
set opt(tr)            	./trace.tr
set opt(rp)             dsr            ;# routing protocol script
set opt(lm)             "off"           ;# log movement
set opt(agent)          Agent/DSR


set AgentTrace			OFF
set RouterTrace			OFF
#set MacTrace			OFF
# ======================================================================
Queue/DropTail/PriQueue set Prefer_Routing_Protocols    1

# unity gain, omni-directional antennas
# set up the antennas to be centered in the node and 1.5 meters above it
Antenna/OmniAntenna set X_ 0
Antenna/OmniAntenna set Y_ 0
Antenna/OmniAntenna set Z_ 1.5
Antenna/OmniAntenna set Gt_ 1.0
Antenna/OmniAntenna set Gr_ 1.0

# Initialize the SharedMedia interface with parameters to make
# it work like the 914MHz Lucent WaveLAN DSSS radio interface

# Table of CS and RX thresholds for 550m and 250m for various std for shadowing model (95%Prob)
# std       CS             RX        
# 4     1.39703e-10    6.76161e-10              
# 8     3.07088e-11    1.48631e-10
# 12    6.75028e-12    3.26713e-11


# Conversely, selecting RX = 1.486e-10 and CS = 3.07e-11 the distances we get for various std are
# std       CS-dist      RX-dist
# 4         1173          533
# 8          550          250
# 12         258          118

Phy/WirelessPhy set CPThresh_ 10.0
#Phy/WirelessPhy set CSThresh_ 1.559e-11 ; # This threshold allows CS at 550m for tworayground
#Phy/WirelessPhy set CSThresh_ 3.071e-11 ; # This threshold allows CS at 550m, std=8 for shadowing
#Phy/WirelessPhy set RXThresh_ 3.652e-10 ; # This threshold allows RX at 250m for tworayground
#Phy/WirelessPhy set RXThresh_ 1.486e-10 ; # This threshold allows RX at 250m, std=8 for shadowing


Phy/WirelessPhy set CSThresh_ 6.373e-10 ;
Phy/WirelessPhy set RXThresh_ 3.0858e-9 ; # This threshold allows RX at 250m, calculated using Freespace


Phy/WirelessPhy set Rb_ 2*1e6
Phy/WirelessPhy set Pt_ 0.2818
Phy/WirelessPhy set freq_ 914e+6 
Phy/WirelessPhy set L_ 1.0




# ======================================================================

proc getopt {argc argv} {
    global opt
# These are the possible parameters that can be set from commandline 
# I use a shell script which calles the tcl file with right parameters
    lappend optlist cp nn seed sc stop tr x y MISB_PERC errorRate LOG_DEV misbnum simType otherNodes totalConnections baseX baseY K THRESH JITTER backoff_policy
    
    for {set i 0} {$i < $argc} {incr i} {
	set arg [lindex $argv $i]
	if {[string range $arg 0 0] != "-"} continue
	
	set name [string range $arg 1 end]
	set opt($name) [lindex $argv [expr $i+1]]
    }
}

# ======================================================================
# Main Program
# ======================================================================
getopt $argc $argv

# do the get opt again incase the routing protocol file added some more
# options to look for
getopt $argc $argv

if { $opt(x) == 0 || $opt(y) == 0 } {
    usage $argv0
    exit 1
}


# code for removing unwanted packet headers. This improves simulation speed

foreach prot {
	AODV
  	aSRM 
	CtrMcast 
	Diffusion
	Encap
	HttpInval
        IPinIP 
	IVS
	LDP
	mcastCtrl
	MFTP
	MPLS
	Message
        MIP 
	Ping
	RAP 
	RTP
	Resv 
	rtProtoDV
	rtProtoLS
	SR
  	SRM 
  	SRMEXT
	Snoop
	TCPA
	TFRC
	TFRC_ACK
	TORA
	GAF
	UMP 
	PUSHBACK
	NV
} {
	remove-packet-header $prot
}



#if {$opt(seed) > 0} 
#{
    #puts "Seeding Random number generator with $opt(seed)\n"
    #ns-random $opt(seed)
#}
    
#
# Initialize Global Variables
#
set ns_		[new Simulator]
set topo	[new Topography]

set tracefd	[open $opt(tr) w]

$topo load_flatgrid $opt(x) $opt(y)

#$ns_ trace-all $tracefd
$ns_ trace-all [open "/dev/null"]

#write the parameters to the trace file
#puts $tracefd "PARAM $opt(MISB_PERC) $opt(stop) $opt(JITTER) $opt(THRESH) $opt(K) $opt(LOG_DEV) $opt(errorRate) $opt(nn)  $opt(rate) "

puts $tracefd "NODES [expr $opt(nn)-1] $opt(MISB_PERC) $opt(stop) $opt(JITTER) $opt(THRESH) $opt(K) $opt(LOG_DEV) $opt(errorRate)  $opt(rate) "

#
# Create God
#
#global god	
set god_ [create-god [expr $opt(nn) + $opt(otherNodes)] ]


set chan1 [new $opt(chan)]

set prop_       [new $opt(prop)]

# PROPERTIES OF SHADOWING MODEL
if { $opt(prop) == "Propagation/Shadowing" } {
    $prop_ set pathlossExp_ 2.0
    $prop_ set std_db_ $opt(LOG_DEV)
    $prop_ set dist0_ 1.0
    $prop_ seed predef 0
}




proc UniformErrProc {} {
    global opt
    set err [new ErrorModel]
    $err set rate_ $opt(errorRate)
    $err unit bit
    return $err
}


#use below option if two ray ground model nees to be used
# -propType $opt(prop) \ 


$ns_ node-config -adhocRouting DSR \
	-llType $opt(ll) \
	-macType $opt(macmb) \
	-ifqType $opt(ifq) \
	-ifqLen $opt(ifqlen) \
	-antType $opt(ant) \
	-propInstance $prop_ \
	-phyType $opt(netif) \
	-channel $chan1 \
	-topoInstance $topo \
	-macTrace ON  \
	-AgentTrace OFF \
#	-errProc  UniformErrProc \
        -IncomingErrProc UniformErrProc \
	-RouterTrace OFF 



#Functions for setting up protocol parameters, monitor, backoff agents 


Node/MobileNode instproc setbackoff {num nodeID MISB_PERC} {
    $self instvar mac_ nifs_
    $mac_(0)  backoff-policy $num $nodeID $MISB_PERC
}



Node/MobileNode instproc setmonitor {JITTER THRESH K} {
    $self instvar mac_ nifs_
    $mac_(0) set-monitor $JITTER $THRESH $K
}

Node/MobileNode instproc setbackoffsender {dummy} {
    $self instvar mac_ nifs_
    $mac_(0) set-backoff-sender $dummy
}

Node/MobileNode instproc setbackoffreceiver {dummy} {
    $self instvar mac_ nifs_
    $mac_(0) set-backoff-receiver $dummy
}

# This function sets up trace file to not include all the usual trace data
# Only data I really need is added. Reduces trace file sizes from 300+ MB
# to a reasonably small summary of what I need
Node/MobileNode instproc my-mobility-trace { ttype atype } {
        global tracefd
	set ns [Simulator instance]
#        set tracefd [$ns get-ns-traceall]
        if { $tracefd == "" } {
	        puts "Warning: You have not defined you tracefile yet!"
	        puts "Please use trace-all command to define it."
		return ""
	}
	set T [new CMUTrace/$ttype $atype]
	$T newtrace [Simulator set WirelessNewTrace_]
	$T target [$ns nullagent]
	$T attach $tracefd
        $T set src_ [$self id]
        $T node $self
	return $T
}

Node/MobileNode instproc set-log {} {
    $self instvar mac_ nifs_
    set sndT [$self my-mobility-trace Recv "MAC"]
    $mac_(0) misb-target $sndT
}



set cbr_cnt 0

proc create-cbr-connection { src dst rate} {
    global cbr_cnt opt ns_ node_ source sink source_id

    # for tracking output later
    set source_id($cbr_cnt) $src

    set source($cbr_cnt)  [new Agent/UDP]
    $ns_ attach-agent $node_($src) $source($cbr_cnt)
    
    set sink($cbr_cnt)    [new Agent/LossMonitor]
    $ns_ attach-agent $node_($dst) $sink($cbr_cnt)
 
    set cbr($cbr_cnt)     [new Application/Traffic/CBR]
    $cbr($cbr_cnt) set packetSize_ 512
    $cbr($cbr_cnt) set rate_ $rate
    $cbr($cbr_cnt) attach-agent $source($cbr_cnt)
    $cbr($cbr_cnt) set random_ 1
    $ns_ connect $source($cbr_cnt) $sink($cbr_cnt)
    
    $ns_ at $opt(start) "$cbr($cbr_cnt) start"

    incr cbr_cnt
}




set default_rng [new RNG]
$default_rng seed predef $opt(seed)
$default_rng default


set god_ [God instance]


if { $opt(networkType) == "BASE" } {

    puts "BASE STATION SCHEME"
    
    # create the other nodes and set up their locations
    
    
    for {set i 0 } {$i < $opt(otherNodes) } {incr i} {
	set node_($i) [$ns_ node] 
	$node_($i) random-motion 0
    }
    
# The following two lines allow for the possibility of having 0,1 or 2 background flows

# Comment this out if u want first background flow to be OFF
    $node_(0) set X_ 250
    $node_(0) set Y_ 350
    $node_(1) set X_ 250
    $node_(1) set Y_ 350
    create-cbr-connection 0 1 $opt(otherRate)   

# Enable this if u want the second background flow to be ON
    #$node_(2) set X_ 1250
    #$node_(2) set Y_ 350
    #$node_(3) set X_ 1250
    #$node_(3) set Y_ 350
    #create-cbr-connection 2 3 $opt(otherRate)



    # Create the base station
    set nodeNum [expr $opt(otherNodes) ]
    
    set node_($nodeNum) [$ns_ node]
    $node_($nodeNum)    random-motion 0
    $node_($nodeNum) set-log
    
    if { $opt(simType) != "SIMPLE" } {
	$node_($nodeNum)    setmonitor $opt(JITTER) $opt(THRESH) $opt(K)	
    }

    # sending 0 as parameter disables Misbehavior correction

    if { $opt(simType) == "NO_MISB" || $opt(simType) == "MISB_NOCORRECT" } {
	$node_($nodeNum)    setbackoffsender 0
    }
    if { $opt(simType) == "MISB_CORRECT" } {
	$node_($nodeNum)    setbackoffsender 1
    }		
    
    $node_($nodeNum) set X_ $opt(baseX)
    $node_($nodeNum) set Y_ $opt(baseY)
    
    
    # this creates n-1 nodes around the Basestation in a circle around the BS
    
    set theta 0
    set theta_incr [expr 360/ ( $opt(nn) - 1) ]
    
    for {set i [expr $nodeNum + 1] } {$i < [expr $nodeNum + $opt(nn)] } {incr i} {
	
	set node_($i) [$ns_ node]
	$node_($i)    random-motion 0
	$node_($i) set-log
	puts $tracefd "SENDER $i"
	
	# Create a backoffreceiver object
	$node_($i)    setbackoffreceiver 0
	
	$node_($i) set X_ [expr $opt(baseX) + $opt(distance) * cos($theta)]
	$node_($i) set Y_ [expr $opt(baseY) + $opt(distance) * sin($theta)]
	set theta [expr $theta + $theta_incr]
    
	$god_ set-dist $nodeNum $i 1 
    }
    
    

    #set up the misbehaving nodes
    if { $opt(simType) == "MISB_NOCORRECT" || $opt(simType) == "MISB_CORRECT" || $opt(simType) == "SIMPLE"} {
	set rng [new RNG]
	$rng seed $opt(seed)
	
	set u [new RandomVariable/Uniform]
	$u set min_ 0
	$u set max_ 100
	$u use-rng $rng
	set misbCount 0
	set selectProb [expr 100 * $opt(misbnum) / ( $opt(nn) - 1)]
	
	for {set k [expr $nodeNum + 1]} {$k < [expr $nodeNum + $opt(nn)] } {incr k} {
	    set status($k) 0
	}
	
	while { $misbCount < $opt(misbnum) } {
	    for {set k [expr $nodeNum + 1]} {$k < [expr $nodeNum + $opt(nn)] } {incr k} {
		
		set x [$u value]
		if { $x < $selectProb } {
		    if { $status($k) == 0 } {
			set status($k) 1
			set misbNode $k
			$node_($misbNode) setbackoff $opt(backoff_policy) $misbNode $opt(MISB_PERC) 
			puts "Node $misbNode is the misbehaving node"
			puts $tracefd "MBN $misbNode"
			
			incr misbCount
		    }
		}
		if { $misbCount == $opt(misbnum) } {
		    break
		}
	    }
	}
	
    }
    
    
  
    # set up the cbr traffic flows 
    set nodeNum [expr $opt(otherNodes) ]
    for {set i [expr $nodeNum + 1]} {$i < [expr $nodeNum + $opt(nn)] } {incr i} {
	
	create-cbr-connection $i $nodeNum $opt(rate)
	
    }
    
# end of if for base station  
} 


#
# Tell all the nodes when the simulation ends
#
for {set i 0} {$i < [expr $opt(nn)  + $opt(otherNodes)] } {incr i} {
    $ns_ at $opt(stop).0 "$node_($i) reset";
}

$ns_ at $opt(stop).1 "stop"
$ns_ at $opt(stop).100001 " $ns_ halt"


proc stop {} {
    global ns_ tracefd opt sink  cbr_cnt source_id
#    $ns_ flush-trace
 
# Write statistics to trace file
    for { set i 0} { $i < $cbr_cnt } {incr i} {
	puts "$source_id($i) [$sink($i) set npkts_]" 
        puts $tracefd "rp $source_id($i) [$sink($i) set npkts_]"
    }
   close $tracefd
  
}


$ns_ run
















