/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*-
 *
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * Ported from CMU/Monarch's code, nov'98 -Padma.
 * wireless-mac-802_11.h
 * Pradeep Kyasanur, March 2003: New class for simulating MAC layer misbehavior, derives code
 * from mac-802_11.h file
 */

#ifndef ns_mac_80211mb_h
#define ns_mac_80211mb_h

#include "mac-timersmb.h"
#include "marshall.h" 
#include "mac-802_11.h" 
#include <map.h>

// Added by Pradeep on 05/21
#include <trace.h>

class Monitor; 
class BackoffReceiver; 
class BackoffSender;
/* ======================================================================
   The actual 802.11 MAC class.
   ====================================================================== */
class Mac802_11mb : public Mac {
	friend class DeferTimermb;
	friend class BackoffTimermb;
	friend class IFTimermb;
	friend class NavTimermb;
	friend class RxTimermb;
	friend class TxTimermb;
public:
	Mac802_11mb(PHY_MIB* p, MAC_MIB *m);
	void		recv(Packet *p, Handler *h);
	inline int	hdr_dst(char* hdr, int dst = -2);
	inline int	hdr_src(char* hdr, int src = -2);
	inline int	hdr_type(char* hdr, u_int16_t type = 0); 
	inline int	is_idle(void); 

	// 3/12/03 Pradeep Kyasanur: Additional function to support modifed idle slot measurement
	inline int	is_idle_modified(void); 
	inline void     set_modified_nav(u_int16_t us);

	inline int getCW(void) { return cw_;}
	inline int getNodeId(void) { return index_;}
protected:
	void	backoffHandler(void);
	void	deferHandler(void);
	void	navHandler(void);
	void	recvHandler(void);
	void	sendHandler(void);
	void	txHandler(void);

private:
	int		command(int argc, const char*const* argv);

	/*
	 * Called by the timers.
	 */
	void		recv_timer(void);
	void		send_timer(void);
	int		check_pktCTRL();
	int		check_pktRTS();
	int		check_pktTx();

	/*
	 * Packet Transmission Functions.
	 */
	void	send(Packet *p, Handler *h);
	void 	sendRTS(int dst);
	void	sendCTS(int dst, double duration);
	void	sendACK(int dst);
	void	sendDATA(Packet *p);
	void	RetransmitRTS();
	void	RetransmitDATA();

	/*
	 * Packet Reception Functions.
	 */
	void	recvRTS(Packet *p);
	void	recvCTS(Packet *p);
	void	recvACK(Packet *p);
	void	recvDATA(Packet *p);

	void		capture(Packet *p);
	void		collision(Packet *p);
	void		discard(Packet *p, const char* why);
	void		rx_resume(void);
	void		tx_resume(void);

	

	/*
	 * Debugging Functions.
	 */
	void		trace_pkt(Packet *p);
	void		dump(char* fname);

	inline int initialized() {
		return (phymib_ && macmib_ && cache_ && logtarget_ && 
			Mac::initialized());
	}
	void mac_log(Packet *p) {
		logtarget_->recv(p, (Handler*) 0);
	}
	inline double TX_Time(Packet *p) {
		double t = DATA_Time((HDR_CMN(p))->size());
		if(t < 0.0) {
			drop(p, "XXX");
			exit(1);
		}
		return t;
	}
	inline void inc_cw() {
		cw_ = (cw_ << 1) + 1;
		if(cw_ > phymib_->CWMax)
			cw_ = phymib_->CWMax;
	}
	inline void rst_cw() { cw_ = phymib_->CWMin; }
	inline u_int16_t usec(double t) {
		u_int16_t us = (u_int16_t)ceil(t *= 1e6);
		return us;
	}
	inline void set_nav(u_int16_t us) {
		double now = Scheduler::instance().clock();
		double t = us * 1e-6;

		if((now + t) > nav_) {
			nav_ = now + t;
			if(mhNav_.busy())
				mhNav_.stop();
			mhNav_.start(t);
		}
	}
 
 

public:
	PHY_MIB		*phymib_;
	MAC_MIB		*macmib_;
 
	// Pradeep Kyasanur: to provide access to backoff timer 
	inline BackoffTimermb* getBackoffTimer(void) 
	{ 
		return &mhBackoff_; 
	}
	
	void trace(char* fmt, ...);
private:
	/*
	 * Mac Timers
	 */
	IFTimermb		mhIF_;		// interface timer
	NavTimermb	    mhNav_;		// NAV timer
	RxTimermb		mhRecv_;		// incoming packets
	TxTimermb		mhSend_;		// outgoing packets

	DeferTimermb	mhDefer_;	// defer timer
	BackoffTimermb	mhBackoff_;	// backoff timer

	/* ============================================================
	   Internal MAC State
	   ============================================================ */
	double		nav_;		// Network Allocation Vector
        
	// 3/12/03 Pradeep Kyasanur: Added a new NAV variable for a modified NAV
        double          modified_nav_;  // Modified Network Allocation Vector

	MacState	rx_state_;	// incoming state (MAC_RECV or MAC_IDLE)
	MacState	tx_state_;	// outgoint state
	int		tx_active_;	// transmitter is ACTIVE

	Packet		*pktRTS_;	// outgoing RTS packet
	Packet		*pktCTRL_;	// outgoing non-RTS packet

	u_int32_t	cw_;		// Contention Window
	u_int32_t	ssrc_;		// STA Short Retry Count
	u_int32_t	slrc_;		// STA Long Retry Count
	double		sifs_;		// Short Interface Space
	double		pifs_;		// PCF Interframe Space
	double		difs_;		// DCF Interframe Space
	double		eifs_;		// Extended Interframe Space
	double		tx_sifs_;
	double		tx_pifs_;
	double		tx_difs_;

	int		min_frame_len_;

	Trace *	logtarget_; // changed NsObject to Trace by Pradeep on 05/21
	Trace * misbtarget_; // added by Pradeep Kyasanur 11/11

	/* ============================================================
	   Duplicate Detection state
	   ============================================================ */
	u_int16_t	sta_seqno_;	// next seqno that I'll use
	int		cache_node_count_;
	Host		*cache_; 
 
	/* Monitor object that is provided with data to Monitor if set */ 
	Monitor * monitor_; 
	BackoffSender * backoffSender; 
	BackoffReceiver * backoffReceiver;


	
	
};

#endif /* __mac_80211mb_h__ */

// ----------- NEW CODE WRITTEN BY PRADEEP KYASANUR ---------------

/* The declaration for the Monitor class  which will detect the misbehavior 
 
   Monitor class will keep track of the number of Idleslots seen by the node. 
   It also tracks the number of idleslots other nodes wait between transmissions 
   using the differences in the Idleslotcounter values. This information can be used  
   in identifying deviations from expected behavior and use it to detect misbehavior 
 
*/ 
 
class NodeInfo 
{ 
public: 

	NodeInfo(u_int32_t nodeId,int THRESH) 
	{ 
		id=nodeId; 
		lastCount=0; 
		givenBackoff=-1; 
		oldBackoff=-1;
		remainingBackoff=0;
		miscount=0;
		windowIndex=0;
		windowFull=0;
		window=new int[THRESH];
		for(int i=0;i<THRESH;i++)
			window[i]=0;
	} 
 
	u_int32_t id;
	
	int lastCount;
	int givenBackoff;
	int oldBackoff;
	int remainingBackoff;
	int *window;
	int windowIndex;
	int windowFull;
	int miscount;
};
 
 
class Monitor  
{ 
 
public: 
	Monitor(Mac802_11mb *mac,double jitter,int thresh,int k) : mac(mac) 
	{ 
		idleCounter = new IdleSlotCounter(mac); 
		idleCounter->start(mac->phymib_->SlotTime); 
		THRESH=thresh;
		K=k;
		jitter_alpha=jitter;
		alpha=0.9;
		averageBackoff=0;
		averageAttempt=0;
		prevCollCount=0;
	} 
 
	void recv(const Packet *p); // capture any type of packet received from MAC to monitor the traffic 
	void updateBackoff(u_int32_t dst,int backoff); // note down value of backoff when sending 
	void updateLastCount(const Packet *p); // note down when last ACK was sent to a node 
	void updateLastCount(u_int32_t dst); // when any packet is overheard, update last count for that node
	void restoreBackoff(const Packet *p); // to restore backoff value when a CTS is lost 
	void restoreBackoff(u_int32_t dst); // to restore backoff value when a CTS is lost 
	map<const u_int32_t,NodeInfo *> nodeTable; 
		int THRESH;
	int K;
	double jitter_alpha;

	/* parameters for estimation successful transmission probability */

        double alpha;
	double averageBackoff;
	double averageAttempt;
	int   prevCollCount;


private : 
	Mac802_11mb * mac; 
	IdleSlotCounter *idleCounter; 
	
	 	/* Protocol parameters added by Pradeep Kyasanur */
 	

	 
 
}; 
 
 
// This class is in receiver and computes and sends the backoff values to be used for transmitting packets to others 
class BackoffSender 
{ 
 
public: 
	BackoffSender(Mac802_11mb *mac,Monitor *mon) : mac(mac),mon(mon)  
	{
		CW=32; 
	}
 

	virtual u_int16_t getBackoff(u_int32_t dst); // get the next backoff value for the destination 
	 
	~BackoffSender()
	{
		mac=0;
		mon=0;
	}
private: 
	Mac802_11mb * mac;
	Monitor *mon;
	int CW; 
	map<const u_int32_t,int> nodeTable;  // this stores the backoff to send for each node 
}; 
 
class BackoffReceiver 
{ 
public: 
	BackoffReceiver(Mac802_11mb *mac) : mac(mac) 
	{ 
		misb = -1;
		dec = 0;
	} 
 
	virtual void setBackoff(Packet *p); //set this value as the backoff to be used for next tranmission 
	int getBackoff(u_int32_t dst); //  get the backoff value to be used for a particular destination 
	u_int16_t getAttemptNumber(u_int32_t dst); 
	void  resetAttemptNumber(u_int32_t dst);
	int misb;
	int dec;
	BackoffTimermb * backoffTimer;
private: 
	Mac802_11mb *mac; 
	map<const u_int32_t,int> nodeTable; // this stores the backoff value to use for each destination
	map<const u_int32_t,int> curTable; // this stores the current value of backoff being used
        map<const u_int16_t,int> attemptNum; // this stores the number of times retransmission has been made 


}; 
