/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 *plot 'population' with boxes





 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Ported from CMU/Monarch's code, nov'98 -Padma.*/
/* Pradeep Kyasanur, March 2003: New timers for simulating MAC layer misbehavior added, code
 * is derived from mac-timers.cc
 */

#include <delay.h>
#include <connector.h>
#include <packet.h>
#include <random.h>
 
// #define DEBUG
//#include <debug.h>
#include <arp.h>
#include <ll.h>
#include <mac.h>
#include <mac-timersmb.h>
#include <mac-802_11mb.h>
#include <mac-802_11.h> 


/*
 * Force timers to expire on slottime boundries.
 */
// #define USE_SLOT_TIME

#define ROUND_TIME()	\
	{								\
		assert(slottime);					\
		double rmd = remainder(s.clock() + rtime, slottime);	\
		if(rmd > 0.0)						\
			rtime += (slottime - rmd);			\
		else							\
			rtime += (-rmd);				\
	}


/* ======================================================================
   Timers
   ====================================================================== */

void
MacTimermb::start(double time)
{
	Scheduler &s = Scheduler::instance();
	assert(busy_ == 0);

	busy_ = 1;
	paused_ = 0;
	stime = s.clock();
	rtime = time;
	assert(rtime >= 0.0);


	s.schedule(this, &intr, rtime);
}

void
MacTimermb::stop(void)
{
	Scheduler &s = Scheduler::instance();

	assert(busy_);

	if(paused_ == 0)
		s.cancel(&intr);

	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;
}

/* ======================================================================
   Defer Timer
   ====================================================================== */
void
DeferTimermb::start(double time)
{
	Scheduler &s = Scheduler::instance();

	assert(busy_ == 0);

	busy_ = 1;
	paused_ = 0;
	stime = s.clock();
	rtime = time;
#ifdef USE_SLOT_TIME
	ROUND_TIME();
#endif
	assert(rtime >= 0.0);

	s.schedule(this, &intr, rtime);
}


void    
DeferTimermb::handle(Event *)
{       
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;



	mac->deferHandler();
}

/* ======================================================================
   NAV Timer
   ====================================================================== */
void    
NavTimermb::handle(Event *)
{       
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;

	mac->navHandler();
}


/* ======================================================================
   Receive Timer
   ====================================================================== */
void    
RxTimermb::handle(Event *)
{       
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;

	mac->recvHandler();
}


/* ======================================================================
   Send Timer
   ====================================================================== */
void    
TxTimermb::handle(Event *)
{       
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;



	mac->sendHandler();
}


/* ======================================================================
   Interface Timer
   ====================================================================== */
void
IFTimermb::handle(Event *)
{
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;

	mac->txHandler();
}


/* ======================================================================
   Backoff Timer
   ====================================================================== */
 
void  
BackoffTimermb::set_backoff_policy(int policy,int nodeID)
{
	backoff_policy=policy; 
	if(policy!=0)  // Misbehavior is starting
		mac->trace("MBS %f %d\n",(Scheduler::instance()).clock(),nodeID);
	else
		mac->trace("MBE %f %d\n",(Scheduler::instance()).clock(),nodeID);
}


void 
BackoffTimermb :: set_misbehavior_percentage(int misb)
{
	MISB_PERC=misb;
}	
   
/*void
BackoffTimermb::setBackoff(int backoff) 
{ 
	nextSlot=backoff; 
} 
*/ 
 
void
BackoffTimermb::handle(Event *)
{
	busy_ = 0;
	paused_ = 0;
	rslots = 0;
	difs_wait = 0.0;

	mac->backoffHandler();
}
 

void
BackoffTimermb::start(int backoff,int idle)
{
	
	
	assert(busy_ == 0);

	busy_ = 1;
	paused_ = 0;
	backoff_stop=0;
	
/*Pradeep Kyasanur:  Calculate the backoff based on the policy selected */ 
	switch( backoff_policy ) 
		{ 
		case 0: // Normal 802.11 scheme 
			   rslots=backoff; 
		       break; 
 
		case 1: // Misbehaving, set backoff to misb_perc always 
			  rslots = MISB_PERC; 
			  break; 
		case 2: // Misbehaving, set backoff to actual - actual * PERCENT / 100 
			  rslots=(int)(backoff * MISB_PERC / 100); 
			  //			  printf("The misbehavior is %d with CW = %d at %f\n",rslots, mac->getCW(),(Scheduler::instance()).clock());
			  rslots = rslots > 0 ? rslots : 1; 
			     
			  break; 
		} 
 

#ifdef USE_SLOT_TIME
	ROUND_TIME();
#endif
	difs_wait = 0.0;

	if(idle == 0)
		paused_ = 1;
	else 
	{ 
		if(rslots>0) 
			stimer->start(mac->phymib_->SlotTime); 
		else 
		{ 
			rslots =1; 
			stimer->start(0); 
		} 
	}
}


void
BackoffTimermb::pause()
{
	
	paused_ = 1;
	stimer->stop(); 
	
}

void
BackoffTimermb::resume(double difs)
{

	assert(busy_ && paused_);

	paused_ = 0;
	backoff_stop=0;

	/*
	 * The media should be idle for DIFS time before we start
	 * decrementing the counter, so I add difs time in here.
	 */
	difs_wait = difs;
	/*
#ifdef USE_SLOT_TIME
	ROUND_TIME();
#endif
	*/
	stimer->start(mac->phymib_->SlotTime+difs_wait);
} 
 

 
 
void BackoffTimermb::countdownHandler(void) 
{ 
  if(backoff_stop !=1) 
	rslots --; 

  if(rslots <= 0) 
	  this->handle(0); 
  else  // continue if the medium is not busy 
  { 
	  if(mac->is_idle()) 
	  { 

		  if(backoff_stop==0)
			  stimer->start(mac->phymib_->SlotTime); 
		  else
			  stimer->start(mac->phymib_->SlotTime + DSSS_SIFSTime + 2*mac->phymib_->SlotTime ); 
		  backoff_stop=0; 
	  } 
	  else 
	  {  // wait for slot and difs time and see it anything changes  
		  // difs = sifs + 2slottime 
          backoff_stop=1; 
		  stimer->start(mac->phymib_->SlotTime ); 
	  } 
  } 
} 
 
 
 
/* Slot timer functions */ 

 
void 
SlotTimermb::start(double time) 
{ 
	Scheduler &s = Scheduler::instance(); 
	s.schedule(this, &intr, time); 
	 
} 
 
void 
SlotTimermb::stop(void) 
{ 
	Scheduler &s = Scheduler::instance(); 
    s.cancel(&intr); 
} 
 
 
 
void     
SlotTimermb::handle(Event *) 
{        
	backoff_->countdownHandler(); 
}
 

/* IdleSlotCounter is used for Monitoring purposes */ 
 
void IdleSlotCounter :: handle(Event *e) 
{ 

	
	/* Pradeep Kyasanur:
	 * If modified backoff countdown behavior is desired (as described in journal paper), then
	 * compile ns by enabling is_idle_modified() function, which is currently commented out.
	 * Other is_idle() function uses 802.11 like backoff countdown
	 */
	
	//if(mac_->is_idle_modified())
	if(mac_->is_idle())
		{
		
			if(deferValue==0)
				{
					idleCount++;
					curIdleCount++;
					start(mac_->phymib_->SlotTime); 
				}
			else
				{
					deferValue--;
					start(mac_->phymib_->SlotTime); 
				}
		}
	else 
		{
			if(curIdleCount!=0){
				collProbability=(1-alpha)*collProbability + alpha*curIdleCount;
				collCount++;
				curIdleCount=0;
			}
			deferValue = 2;        // when channel goes idle we have to wait for DIFS time which is roughly 2 slots
			start(mac_->phymib_->SlotTime); 
		}
} 
	 
double IdleSlotCounter :: getCollisionProbability(void)
{
	return collProbability;
}

void IdleSlotCounter :: start(double time) 
{ 
	Scheduler &s=Scheduler::instance(); 
	s.schedule(this,&intr,time); 
} 
	 
void IdleSlotCounter :: stop(void) 
{ 
	Scheduler &s=Scheduler::instance(); 
	s.cancel(&intr); 
} 
 
int IdleSlotCounter :: getCurrentCount(void) 
{ 
	return idleCount; 
} 
 
int IdleSlotCounter :: getCurrentCollCount(void)
{
	return collCount;
}
