
/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Ported from CMU/Monarch's code, nov'98 -Padma.*/
/* Pradeep Kyasanur, March 2003: New timers for simulating MAC layer misbehavior added, code
 * is derived from mac-timers.h
 */
#ifndef __mac_timersmb_h__
#define __mac_timersmb_h__

/* ======================================================================
   Timers
   ====================================================================== */
class Mac802_11mb;

class MacTimermb : public Handler {
public:
	MacTimermb(Mac802_11mb* m, double s = 0) : mac(m), slottime(s) {
		busy_ = paused_ = 0; stime = rtime = 0.0;
	}

	virtual void handle(Event *e) = 0;

	virtual void start(double time);
	virtual void stop(void);
	virtual void pause(void) { assert(0); }
	virtual void resume(void) { assert(0); }

	inline int busy(void) { return busy_; }
	inline int paused(void) { return paused_; }
	inline double expire(void) {
		return ((stime + rtime) - Scheduler::instance().clock());
	}

protected:
	Mac802_11mb	*mac;
	int		busy_;
	int		paused_;
	Event		intr;
	double		stime;	// start time
	double		rtime;	// remaining time
	double		slottime;
};


/* Added by Pradeep Kyasanur to count down the backoff slots */ 
 
class BackoffTimermb; 
 
class SlotTimermb : public Handler { 
public: 
	SlotTimermb(BackoffTimermb *b): busy_(0) { backoff_=b;} 
	void start(double time); 
	void stop(void); 
	void handle(Event *e); 
private: 
	BackoffTimermb *backoff_; 
	Event intr; 
	int busy_; 
}; 
 

class BackoffTimermb : public MacTimermb {
public:
	BackoffTimermb(Mac802_11mb *m, double s) : MacTimermb(m, s), difs_wait(0.0), backoff_policy(0),rslots(0),backoff_stop(0)  
	{ 
		stimer = new SlotTimermb(this); 
		MISB_PERC=0;
	//	nextSlot = (Random::random() % 32) ; 
	}

	void	start(int backoff, int idle);
	void	handle(Event *e);
	void	pause(void);
	void	resume(double difs); 
	void    set_backoff_policy(int policy,int nodeID);  
	void    countdownHandler(void);
	void    set_misbehavior_percentage(int);
//	void    setBackoff(int backoff);
private:
	double	difs_wait; 
	int rslots; 
	int backoff_stop; 
	int backoff_policy; 
	//int nextSlot; 
	SlotTimermb *stimer;
	int MISB_PERC;
};

class DeferTimermb : public MacTimermb {
public:
	DeferTimermb(Mac802_11mb *m, double s) : MacTimermb(m,s) {}

	void	start(double);
	void	handle(Event *e);
};

class IFTimermb : public MacTimermb {
public:
	IFTimermb(Mac802_11mb *m) : MacTimermb(m) {}

	void	handle(Event *e);
};

class NavTimermb : public MacTimermb {
public:
	NavTimermb(Mac802_11mb *m) : MacTimermb(m) {}

	void	handle(Event *e);
};

class RxTimermb : public MacTimermb {
public:
	RxTimermb(Mac802_11mb *m) : MacTimermb(m) {}

	void	handle(Event *e);
};

class TxTimermb : public MacTimermb {
public:
	TxTimermb(Mac802_11mb *m) : MacTimermb(m) {}

	void	handle(Event *e);
};
 
 
/* IdleSlotCounter is used for Monitoring purposes */ 
 
class IdleSlotCounter : public Handler 
{ 
 
public:	 
	IdleSlotCounter(Mac802_11mb *m):idleCount(0),deferValue(1),alpha(0.1),collProbability(0),curIdleCount(0),collCount(0),mac_(m) {} 
	void handle(Event *e); 
	void start(double time); 
	void stop(void); 
	int getCurrentCount(void); 
	double getCollisionProbability(void);
	int getCurrentCollCount(void);
 
private: 
	int idleCount; 
	int deferValue;
	int curIdleCount;

	int collCount;
	double alpha;
	double collProbability;
	Event intr; 
	Mac802_11mb *mac_; 
	 
}; 
 

#endif /* __mac_timersmb_h__ */

