#!/bin/csh

# Created by Pradeep Kyasanur
# This shell file is a sample script for generating multiple runs, etc.
# Last Modified: July 24, 2003
#
# This file sets up the simulation runs
#




#Initialization
unset noclobber

#Purpose of this simulation
#set reason = "This simulation is being run to validate adaptive misbehavior version when there is no misbehavior"
set reason = "This simulation is being run to plot simulation results for adaptive misbehavior, with modified NAV, one flow"

#The name of this file. Used to back up the file
set myself = "sample.csh"

# Name  of the tcl file to use without .tcl extension
set tcl_file = "example"


# set up the output directory for simulation output and graphs
set outdir = "./simout-july22/adapt_modifiedNAV"
set scendir = ./scen/adhoc/techrep-aug23
set perldir = ./perl-scripts
set tracedir = ./trace

#Executable to use - > ns and its path

set exec_file = "./ns-versions/ns-july22-adaptive-modifiedNAV"

#scenario file base name for ad hoc simulations
set scen_file = "scen-ADHOC-nn-50-1500x750-"

# set up position of base station
set baseX = 750
set baseY = 350


#set up parameters

# number of misbehaving nodes
set misbnum = 1

#set up the outer loops to create individual output file

# set the backoff policy - 3 = adaptive misbehavior

set backoff = 3 

# K SHOULD BE SMALLER THAN THRESHOLD

foreach THRESH (5 1)

if ( $THRESH == 5 ) then
 set K = 20 
else if ( $THRESH == 10 ) then
 set K = 40
else
 set K = 0
endif

#set THRESH = 2
set JITTER = 1.0

set error = 0
set logdev = 1

#NN
foreach nn (9)
#foreach misb (100)

#NN
#set output_dir  = "$outdir/NOADAPT,nn=$nn,Thresh=$THRESH,K=$K,Alpha=$JITTER,stdev=$logdev"
set output_dir  = "$outdir/ONE,ADAPT,MODIFIED_NAV,nn=$nn,Thresh=$THRESH,K=$K,Alpha=$JITTER,stdev=$logdev"
#set output_dir  = "$outdir/ONE,MODIFIED_NAV,nn=$nn,Thresh=$THRESH,K=$K,Alpha=$JITTER,stdev=$logdev"
mkdir $output_dir

foreach simtype (MISB_CORRECT)
#foreach simtype (MISB_CORRECT MISB_NOCORRECT)
#foreach simtype (NO_MISB)
#foreach simtype (SIMPLE)


# The output file name
#set output_file = "Type=$simtype,nn=$nn,othernode=24,bsat$baseX-$baseY,Thresh=$THRESH,K=$K,Alpha=$JITTER,stdev=$logdev.dat"

#set output_file = "Type=$simtype,nn=Varying,othernode=24,bsat$baseX-$baseY,Thresh=$THRESH,K=$K,Alpha=$JITTER,stdev=$logdev.dat"


set logdir = "$output_dir/log"
mkdir $logdir


 set absSum  = "$output_dir/$simtype,idletput.dat"
 set detSum =  "$logdir/$simtype,idletput,DET,SUM.dat"
 set idleSum =  "$output_dir/$simtype,idleslot.dat"
 

 rm $absSum
 rm $detSum
 rm $idleSum

 set baseNode = 2
 set totalNode = 8
 

#Remove old version of output file , comment the next line and the line for writing comments to output file if data has to be appended


# Write any comment to the output directory
echo `date` >> "$output_dir/README.txt"
echo "# Simulation output for backoff_policy = $backoff, Nodes = $nn , Misb $misbnum Error = $error simType = $simtype logdev = $logdev seeds 1-30 1500x750 BASE setup Jitter = $JITTER Thresh = $THRESH K = $K"  >> "$output_dir/README.txt"
echo "ns-version used is $exec_file" >> "$output_dir/README.txt"
echo "PURPOSE=$reason" >> "$output_dir/README.txt"

#Back up the script files used for the simulation
cp $tcl_file.tcl $output_dir
cp $myself    $output_dir

# create a status file which records the result of each run
set status_file = "$output_dir/STATUS"
echo "Status file" > $status_file

#The loop for X axis 
#NN
#foreach nn (2 3 5 9 17 33 65)
#foreach misb (0 5 10 20 30 40 50 60 70 80 90 100)
#foreach misb (0  10 40 60 80 100)
foreach misb (0 1 2 3 5 10 15 20 30 50 75)

  #NN
  set absFile = "$logdir/$simtype,idletput,$misb,ABS.dat"
  set detFile = "$logdir/$simtype,idletput,$misb,DET.dat"
  set idleFile = "$logdir/$simtype,idleslot,$misb.dat"

  #set absFile = "$logdir/$simtype,idletput,$nn,ABS.dat"
  #set detFile = "$logdir/$simtype,idletput,$nn,DET.dat"
  #set idleFile = "$logdir/$simtype,idleslot,$misb.dat"
    
  rm $absFile
  rm $detFile
  rm $idleFile
   
  

   # generate one point of the simulation by averaging runs
  #foreach seed (1 2 3 4 5) 
  foreach seed (1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30)
  #  foreach seed (1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20)   
      echo Starting simulation for Number of nodes = $nn, simtype = $simtype, misb = $misb, seed $seed, K = $K, THRESH = $THRESH >> $status_file

       
      $exec_file $tcl_file.tcl -seed $seed  -tr $tcl_file.tr -errorRate $error  -LOG_DEV $logdev -MISB_PERC $misb  -simType $simtype -misbnum $misbnum -nn $nn -sc $scendir/$scen_file$seed -baseX $baseX -baseY $baseY -K $K -THRESH $THRESH -JITTER $JITTER -backoff_policy $backoff >> $status_file
  
echo "------------" >> $status_file
#     $perldir/channel.pl $tracedir/$tcl_file.output $idleFile $baseNode $totalNode $misb 
# NN
 $perldir/count-new.pl $tcl_file.tr $absFile $detFile $misb    
#     $perldir/count-new.pl $tcl_file.tr $absFile $detFile $nn    


    end



#  set idleCol = `expr $totalNode \* 5`
#  set idleCol = `expr $idleCol + 3`
#  $perldir/sum.pl $idleFile $idleSum $idleCol
  $perldir/sum.pl $absFile  $absSum  9
  $perldir/sum.pl $detFile  $detSum  5 

 
end

# Loop ending for outer loops

end

#end
end

end






































